/*
 * callbacks.c
 */

/*
 * TODO:
 * Give propertybox opening playernames
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "table.h"
#include "gamehandler.h"
#include "dice.h"


gint tmp_player_num = 1;
gboolean spinbutton_flag = FALSE;
GtkWidget *appbar;


void
on_new_game_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget *mbx;

    // No game running?
    if (gamehandler.gameflow_status == GAMEFLOW_UNDEFINED)
	return;
    
    mbx = gnome_message_box_new("Are you sure you want to start a new game?",
				GNOME_MESSAGE_BOX_QUESTION,
				GNOME_STOCK_BUTTON_YES,
				GNOME_STOCK_BUTTON_NO,
				NULL);
    if (!gnome_dialog_run_and_close(GNOME_DIALOG(mbx)))
	gamehandler_new_game();
}


void
on_restart_game_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    gnome_dialog_run_and_close(GNOME_DIALOG(gnome_ok_dialog("Sorry, this feature is not yet implemented!")));
}


void
on_highscore_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    gnome_dialog_run_and_close(GNOME_DIALOG(gnome_ok_dialog("Sorry, this feature is not yet implemented!")));
}


void
on_preferences_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget *pbox;
    pbox = create_propertybox();
    gtk_widget_show(pbox);
}


void
on_info_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    gtk_widget_show(create_about());
}

void
on_mainapp_destroy                     (GtkObject       *object,
                                        gpointer         user_data)
{
    gtk_main_quit();
}

gboolean
on_mainapp_delete_event                (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
#ifdef ASK_QUIT
    GtkWidget *mbx;

    mbx = gnome_message_box_new("Are you sure you want to quit?",
				GNOME_MESSAGE_BOX_QUESTION,
				GNOME_STOCK_BUTTON_YES,
				GNOME_STOCK_BUTTON_NO,
				NULL);
    if (!gnome_dialog_run_and_close(GNOME_DIALOG(mbx)))
    {
	return FALSE;
    }
    else
	return TRUE;
#else
    gtk_main_quit();
    return TRUE;
#endif
}


void
on_propertybox_apply                   (GnomePropertyBox *gnomepropertybox,
                                        gint             arg1,
                                        gpointer         user_data)
{
    GtkEntry *w;

    // g_print("Propertybox page %d\n", arg1);

    g_assert(gnomepropertybox != NULL);
    
    switch (arg1)
    {
    case 0:  // Page 1
	if (gamehandler.player_num != tmp_player_num &&
	    gamehandler.gameflow_status != GAMEFLOW_UNDEFINED)
	{
	    GtkWidget *q;
	    
	    q = gnome_message_box_new("This will restart the current game. Proceed?",
				      GNOME_MESSAGE_BOX_WARNING,
				      GNOME_STOCK_BUTTON_YES,
				      GNOME_STOCK_BUTTON_NO,
				      NULL);
	    if (gnome_dialog_run_and_close(GNOME_DIALOG(q)))
	    {
		GtkSpinButton *sb;
		
		tmp_player_num = gamehandler.player_num;
		sb = GTK_SPIN_BUTTON(lookup_widget(GTK_WIDGET(gnomepropertybox), "spinbutton_playernum"));
		g_assert(sb != NULL);
		gtk_spin_button_set_value(sb, tmp_player_num);
		adjust_playername_entries(GTK_WIDGET(gnomepropertybox));
		return;
	    }
	    else
		gamehandler_new_game();
	}

	w = GTK_ENTRY(lookup_widget(GTK_WIDGET(gnomepropertybox), "playername1"));
	g_assert(w != NULL);
	if (!strcmp(gtk_entry_get_text(w), ""))
	{
	    gamehandler.player_name[0] = "Player 1";
	    gtk_entry_set_text(w, "Player 1");
	    gnome_dialog_run_and_close(GNOME_DIALOG(gnome_warning_dialog("Playername must not be empty.")));
	}
	else
	    gamehandler.player_name[0] = gtk_editable_get_chars(GTK_EDITABLE(w), 0, -1);

	if (tmp_player_num >= 2)
	{
	    w = GTK_ENTRY(lookup_widget(GTK_WIDGET(gnomepropertybox), "playername2"));
	    g_assert(w != NULL);
	    if (!strcmp(gtk_entry_get_text(w), ""))
	    {
		gamehandler.player_name[1] = "Player 2";
		gtk_entry_set_text(w, "Player 2");
		gnome_dialog_run_and_close(GNOME_DIALOG(gnome_warning_dialog("Playername must not be empty.")));
	    }
	    else
		gamehandler.player_name[1] = gtk_editable_get_chars(GTK_EDITABLE(w), 0, -1);
	}

	if (tmp_player_num >= 3)
	{
	    w = GTK_ENTRY(lookup_widget(GTK_WIDGET(gnomepropertybox), "playername3"));
	    g_assert(w != NULL);
	    if (!strcmp(gtk_entry_get_text(w), ""))
	    {
		gamehandler.player_name[2] = "Player 3";
		gtk_entry_set_text(w, "Player 3");
		gnome_dialog_run_and_close(GNOME_DIALOG(gnome_warning_dialog("Playername must not be empty.")));
	    }
	    else
		gamehandler.player_name[2] = gtk_editable_get_chars(GTK_EDITABLE(w), 0, -1);
	}
	
	gamehandler.player_num = tmp_player_num;
	table_set_playernames();

	break;
    case 1:  // Page 2
	break;
    }
}


void
on_propertybox_help                    (GnomePropertyBox *gnomepropertybox,
                                        gint             arg1,
                                        gpointer         user_data)
{
    gnome_dialog_run_and_close(GNOME_DIALOG(gnome_ok_dialog("Sorry, this feature is not yet implemented!")));
}


void
init                                   (GtkWidget       *widget,
                                        gpointer         user_data)
{
    appbar = lookup_widget(widget, "appbar");
    g_assert(appbar != NULL);

    init_gamehandler(widget);
    init_table(user_data);
    init_dice_pixmaps();
}

void
on_buttonDoRoll_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
    if (gamehandler.gameflow_status != GAMEFLOW_ROLLING &&
	gamehandler.gameflow_status != GAMEFLOW_STARTTURN &&
	gamehandler.gameflow_status != GAMEFLOW_UNDEFINED)
    {
	g_print("Not rolling right now.\n");
	statusbar("It is not the time to roll the dice now.");
	return;
    }
    else
    {
	int i, rolls[6];
    
	for (i=0; i<5; i++)
	    rolls[i] = gamehandler_get_random_number();
	
	gamehandler_handle_roll(rolls, GTK_WIDGET(button));
	dice_exchange(GTK_WIDGET(button)->parent);
    }
}


void
on_table_select_row                    (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
    if (gamehandler.gameflow_status != GAMEFLOW_CHOOSING)
    {
	g_print("Not choosing right now. Row = %d\n", row);
	statusbar("It is time to roll the dice.");
	gtk_clist_unselect_row(clist, row, column);
	return;
    }
    g_print("select row %d\n", row);
    // Mark the selection in the table array, check for valid marking
    if (table_add_selection(row))
    {
	g_print("valid selection\n");
	// Remove the unselected values from the table
	table_remove_unselected();
	// Switch gamestatus to rolling again and reset gamehandler
	gamehandler_reset_values();
	// Next player
	gamehandler_next_player();
    }
    else
    {
	g_print("no valid selection\n");
	statusbar("Sorry, that was no valid selection.");
	// Unselect the table selection
	gtk_clist_unselect_row(clist, row, column);
    }	
}


gboolean
select_die                             (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
    if (gamehandler.gameflow_status != GAMEFLOW_ROLLING)
    {
	statusbar("Do the first roll before selecting!");
	g_print("Not rolling right now.\n");
	return TRUE;
    }
    gamehandler_select_die((int)user_data);
    dice_hilite((int)user_data, widget);
    statusbar("Oh, that die looks good!");
    return FALSE;
}

void
on_spinbutton_playernum_changed        (GtkEditable     *editable,
                                        gpointer         user_data)
{
    tmp_player_num = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(editable));;
    adjust_playername_entries(GTK_WIDGET(editable));
    
    if (!spinbutton_flag)
	gnome_property_box_changed(GNOME_PROPERTY_BOX(user_data));
    else
	spinbutton_flag = FALSE;
}

void
adjust_playername_entries(GtkWidget *parent)
{
    GtkWidget *w;

    g_assert(parent != NULL);
    
    // Check field for player 2
    w = lookup_widget(parent, "playername2");
    g_assert(w != NULL);
    gtk_widget_set_sensitive(w, (tmp_player_num >= 2 ? TRUE : FALSE));

    // Check field for player 3
    w = lookup_widget(parent, "playername3");
    g_assert(w != NULL);
    gtk_widget_set_sensitive(w, (tmp_player_num >= 3 ? TRUE : FALSE));
}


void
on_playername_changed                  (GtkEditable     *editable,
                                        gpointer         user_data)
{
    gnome_property_box_changed(GNOME_PROPERTY_BOX(user_data));
}


// Ok, very dirty hack. It would of course be cooler, if the values are instantly
// set properly in the creation, but that would require changes in glades interface.c
// create_propertybox() function.
// To avoid that currently, we use this hack. In a later, final version it might change.
void
adjust_playernames(GtkWidget *parent)
{
    GtkEntry *entry;
    
    // Check field for player 1
    if (strcmp(gamehandler.player_name[0], "Player 1"))
    {
	entry = GTK_ENTRY(lookup_widget(parent, "playername1"));
	g_assert(entry != NULL);
	gtk_entry_set_text(entry, gamehandler.player_name[0]);
    }

    // Check field for player 2
    if (strcmp(gamehandler.player_name[1], "Player 2"))
    {
	entry = GTK_ENTRY(lookup_widget(parent, "playername2"));
	g_assert(entry != NULL);
	gtk_entry_set_text(entry, gamehandler.player_name[1]);
    }

    // Check field for player 1
    if (strcmp(gamehandler.player_name[2], "Player 3"))
    {
	entry = GTK_ENTRY(lookup_widget(parent, "playername3"));
	g_assert(entry != NULL);
	gtk_entry_set_text(entry, gamehandler.player_name[2]);
    }
}


// Very dirty hack II, see above
void
on_spinbutton_playernum_realize        (GtkWidget       *widget,
                                        gpointer         user_data)
{
    if (gamehandler.player_num != 1)
    {
	spinbutton_flag = TRUE;
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), gamehandler.player_num);
    }

    adjust_playernames(widget);
}

void
statusbar(const gchar *txt)
{
    gnome_appbar_set_status(GNOME_APPBAR(appbar), txt);
}
