/*
 * dice.c
 */

/*
 * TODO: When releasing, change support.c pixmap loading to #includes
 *       instead of files and remove pixmap dir from dist. And adjust
 *       Makefile.am by removing install_data_local
 *
 * IMPORTANT: free pixmaps and bitmap pointer !!!
 */

#include "dice.h"
#include "support.h"
#include "gamehandler.h"
#include "pixmaps.h"

GdkPixmap *dice_pixmaps[6];
GdkPixmap *dice_pixmaps_hilite[6];
GdkBitmap *dice_bitmap;

void
init_dice_pixmaps()
{
    int i;
    gchar *die = NULL,
	*die_hilite = NULL;

#ifdef INCLUDE_FROM_FILE
    for (i=0; i<6; i++)
    {
	switch (i)
	{
	case 0: die = DIE1; die_hilite = DIE_HILITE1; break;
	case 1: die = DIE2; die_hilite = DIE_HILITE2; break;
	case 2: die = DIE3; die_hilite = DIE_HILITE3; break;
	case 3: die = DIE4; die_hilite = DIE_HILITE4; break;
	case 4: die = DIE5; die_hilite = DIE_HILITE5; break;
	case 5: die = DIE6; die_hilite = DIE_HILITE6; break;
	}

	if (!(die = gnome_pixmap_file(die)) ||
	    !(die_hilite = gnome_pixmap_file(die_hilite)))
	{
	    g_error("Could not find dice pixmaps! "
		    "Check your installation. Have you installed in /opt/gnome ?\n");
	}

	dice_pixmaps[i] =
	    gdk_pixmap_colormap_create_from_xpm(NULL,
						gtk_widget_get_default_colormap(),
						&dice_bitmap,
						NULL,
						die);
	dice_pixmaps_hilite[i] =
	    gdk_pixmap_colormap_create_from_xpm(NULL,
						gtk_widget_get_default_colormap(),
						&dice_bitmap,
						NULL,
						die_hilite);
    }
#else
    dice_pixmaps[0] =
	gdk_pixmap_colormap_create_from_xpm_d(NULL,
					      gtk_widget_get_default_colormap(),
					      &dice_bitmap,
					      NULL,
					      (gchar**)DIE1);
    dice_pixmaps[1] =
	gdk_pixmap_colormap_create_from_xpm_d(NULL,
					      gtk_widget_get_default_colormap(),
					      &dice_bitmap,
					      NULL,
					      (gchar**)DIE2);
    dice_pixmaps[2] =
	gdk_pixmap_colormap_create_from_xpm_d(NULL,
					      gtk_widget_get_default_colormap(),
					      &dice_bitmap,
					      NULL,
					      (gchar**)DIE3);
    dice_pixmaps[3] =
	gdk_pixmap_colormap_create_from_xpm_d(NULL,
					      gtk_widget_get_default_colormap(),
					      &dice_bitmap,
					      NULL,
					      (gchar**)DIE4);
    dice_pixmaps[4] =
	gdk_pixmap_colormap_create_from_xpm_d(NULL,
					      gtk_widget_get_default_colormap(),
					      &dice_bitmap,
					      NULL,
					      (gchar**)DIE5);
    dice_pixmaps[5] =
	gdk_pixmap_colormap_create_from_xpm_d(NULL,
					      gtk_widget_get_default_colormap(),
					      &dice_bitmap,
					      NULL,
					      (gchar**)DIE6);

    dice_pixmaps_hilite[0] =
	gdk_pixmap_colormap_create_from_xpm_d(NULL,
					      gtk_widget_get_default_colormap(),
					      &dice_bitmap,
					      NULL,
					      (gchar**)DIE_HILITE1);
    dice_pixmaps_hilite[1] =
	gdk_pixmap_colormap_create_from_xpm_d(NULL,
					      gtk_widget_get_default_colormap(),
					      &dice_bitmap,
					      NULL,
					      (gchar**)DIE_HILITE2);
    dice_pixmaps_hilite[2] =
	gdk_pixmap_colormap_create_from_xpm_d(NULL,
					      gtk_widget_get_default_colormap(),
					      &dice_bitmap,
					      NULL,
					      (gchar**)DIE_HILITE3);
    dice_pixmaps_hilite[3] =
	gdk_pixmap_colormap_create_from_xpm_d(NULL,
					      gtk_widget_get_default_colormap(),
					      &dice_bitmap,
					      NULL,
					      (gchar**)DIE_HILITE4);
    dice_pixmaps_hilite[4] =
	gdk_pixmap_colormap_create_from_xpm_d(NULL,
					      gtk_widget_get_default_colormap(),
					      &dice_bitmap,
					      NULL,
					      (gchar**)DIE_HILITE5);
    dice_pixmaps_hilite[5] =
	gdk_pixmap_colormap_create_from_xpm_d(NULL,
					      gtk_widget_get_default_colormap(),
					      &dice_bitmap,
					      NULL,
					      (gchar**)DIE_HILITE6);
#endif
}

gchar *get_pixmap_name(int which)
{
    switch (which)
    {
    case 0: return "pixmap_die1";
    case 1: return "pixmap_die2";
    case 2: return "pixmap_die3";
    case 3: return "pixmap_die4";
    case 4: return "pixmap_die5";
    }
    
    g_assert_not_reached();
    return NULL;
}

void
dice_exchange(GtkWidget *parent)
{
    GtkWidget *w;
    int i;

    for (i=0; i<5; i++)
    {
	w = lookup_widget(parent, get_pixmap_name(i));
	g_assert(w != NULL &&
		 gamehandler.rolls[i] >= 1 && gamehandler.rolls[i] <= 6);
	gtk_pixmap_set(GTK_PIXMAP(w),
		       (!gamehandler.rolls_stored[i] ?
			dice_pixmaps[gamehandler.rolls[i]-1] :
			dice_pixmaps_hilite[gamehandler.rolls[i]-1]),
		       dice_bitmap);
    }
}

void
dice_hilite(int col, GtkWidget *parent)
{
    GtkWidget *w;
    
    w = lookup_widget(parent, get_pixmap_name(col));
    g_assert(w != NULL &&
	     gamehandler.rolls[col] >= 1 && gamehandler.rolls[col] <= 6);
    gtk_pixmap_set(GTK_PIXMAP(w),
		   (!gamehandler.rolls_stored[col] ?
		    dice_pixmaps[gamehandler.rolls[col]-1] :
		    dice_pixmaps_hilite[gamehandler.rolls[col]-1]),
		   dice_bitmap);
}
