/*
 * table.c
 */

/*
 * TODO: g_free table_clist, style_red and style_normal somewhere
 */

#include "table.h"
#include "defines.h"
#include "gamehandler.h"
#include "support.h"
#include "callbacks.h"

int table_entries[TABLE_ROWS][TABLE_COLS];
GtkStyle *style_red, *style_normal;
GtkCList *table_clist;


void
init_table(gpointer *table)
{
    int i, j;
    gchar *txt[4] = {"", "", "", ""};
    gchar *str[TABLE_ROWS] =
	{ "1's", "2's", "3's", "4's", "5's", "6's",
	  "Int.-Sum", "Bonus (35)", "Sum", "3 Doublets", "4 Doublets",
	  "Full House", "Small Street", "Grand Street", "Yatzi",
	  "Chance", "Sum", "Total Sum" };

    g_assert(table != NULL);
    table_clist = GTK_CLIST(table);
    g_assert(table_clist != NULL);

    // Right align the value columns, leave left align for the first column
    for (i=1; i<=TABLE_COLS; i++)
	gtk_clist_set_column_justification(table_clist, i, GTK_JUSTIFY_CENTER);

    // Passive titles
    gtk_clist_column_titles_passive(table_clist);

    // Print the left column texts, leave the other columns free
    for (i=0; i<TABLE_ROWS; i++)
    {
	gtk_clist_append(table_clist, txt);
	gtk_clist_set_text(table_clist, i, 0, str[i]);
    }

    // Fill the table array with dummy flags
    for (i=0; i<TABLE_ROWS; i++)
    {
	for (j=0; j<TABLE_COLS; j++)
	    table_entries[i][j] = -1;
    }

    // Create the style for the red and the one for the normal text
    style_red = create_style_red(GTK_WIDGET(table));
    style_normal = gtk_style_copy(gtk_widget_get_style(GTK_WIDGET(table)));

    table_hilite_player_label();
}

// Reset the table for a new game
void
reset_table()
{
    int i, j;

    g_print("reset table\n");

    // Lock the visual updates
    gtk_clist_freeze(table_clist);

    for (i=0; i<TABLE_ROWS; i++)
    {
	for (j=0; j<TABLE_COLS; j++)
	{
	    gtk_clist_set_text(table_clist, i, j+1, "");
	    table_entries[i][j] = -1;
	}
    }

    // Unlock the visual updates
    gtk_clist_thaw(table_clist);
}

void
table_set_cell(int row, int col, int value, GtkStyle *style)
{
    char buffer[5] = "    ";
    
    g_assert(value >= 0 && table_clist != NULL);
   
    sprintf(buffer, "%4d", value);
    gtk_clist_set_text(table_clist, row, col, buffer);
    gtk_clist_set_cell_style(table_clist, row, col, style);
}

GtkStyle *
create_style_red(GtkWidget *widget)
{
    GdkColor color_red;
    GdkFont *font;
    GtkStyle *style;
    
    style = gtk_style_copy(gtk_widget_get_style(widget));

    // A medium dark red
    color_red.red = 0xdddd;
    color_red.green = 0x0000;
    color_red.blue = 0x0000;
    style->fg[GTK_STATE_NORMAL] = color_red;

    // Bold font - I hope this one is familiar enough on most linux installations. :)
    font = gdk_font_load("-*-helvetica-bold-r-normal--12-120-*-*-*-*-iso8859-1");
    style->font = font;
    
    return style;
}

// Called when the turn of a player is through, to print the possible
// moves into the table
void
table_update_column()
{
    int i,
	counter = 0,
	player = gamehandler.current_player;
    
    g_assert(table_clist != NULL);
    
    // Lock the visual updates
    gtk_clist_freeze(table_clist);

    gtk_clist_unselect_all(table_clist);

    // Update the table

    // The number fields	
    for (i=0; i<6; i++)
    {
	if (table_entries[i][player-1] == -1)
	{
	    if (result.numbers[i])
		table_set_cell(i, player, gamehandler_get_value(i), style_red);
	    else
		table_set_cell(i, player, 0, style_red);
	    counter ++;
	}
    }

    // The doublets
    switch (result.pasch & 24)
    {
    case PASCH5_FLAG:
	if (table_entries[PASCH5][player-1] == -1)
	{
	    table_set_cell(PASCH5, player, PASCH5_VALUE, style_red);
	    counter ++;
	}
    case PASCH4_FLAG:
	if (table_entries[PASCH4][player-1] == -1)
	{
	    table_set_cell(PASCH4, player, gamehandler_get_value(PASCH4), style_red);
	    counter ++;
	}
    case PASCH3_FLAG:
	if (table_entries[PASCH3][player-1] == -1)
	{
	    table_set_cell(PASCH3, player, gamehandler_get_value(PASCH3), style_red);
	    counter ++;
	}
    }

    // The special fields
    switch (result.erg)
    {
    case FULL_HOUSE:
	if (table_entries[FULL_HOUSE][player-1] == -1)
	{
	    table_set_cell(FULL_HOUSE, player, FULL_HOUSE_VALUE, style_red);
	    counter ++;
	}
	break;
    case GRAND_STREET:
	if (table_entries[GRAND_STREET][player-1] == -1)
	{
	    table_set_cell(GRAND_STREET, player, GRAND_STREET_VALUE, style_red);
	    counter ++;
	}
    case SMALL_STREET:
	if (table_entries[SMALL_STREET][player-1] == -1)
	{
	    table_set_cell(SMALL_STREET, player, SMALL_STREET_VALUE, style_red);
	    counter ++;
	}
    }

    // Unfilled fields without value and without button yet
    for (i=PASCH3; i<CHANCE; i++)
    {
	gchar *txt;
	if (table_entries[i][player-1] == -1 &&
	    gtk_clist_get_text(table_clist, i, player, &txt) &&
	    !strcmp(txt, ""))
	{
	    table_set_cell(i, player, 0, style_red);
	    counter ++;
	}
    }
		
    // Chance
    if (table_entries[CHANCE][player-1] == -1)
    {
	table_set_cell(CHANCE, player, gamehandler_get_value(CHANCE), style_red);
	counter ++;
    }

    // Is this player done with his Game?
    if (counter == 1)
    {
	gamehandler.player_is_done[player-1] = TRUE;
	g_print("PLAYER IS DONE WITH GAME\n");
    }

    // Unlock the visual updates
    gtk_clist_thaw(table_clist);
}

// Check is selection is valid, if yes, enter the selection and
// return TRUE, if no, do nothing and return FALSE.
gboolean
table_add_selection(int row)
{
    // Check if selection is valid
    if (table_entries[row][gamehandler.current_player-1] != -1 ||
	row == INTER_SUM ||
	row == BONUS ||
	row == UP_SUM ||
	row == LOW_SUM ||
	row == TOTAL_SUM)
	return FALSE;
    else
    {
	gchar *txt;
	
	gtk_clist_get_text(table_clist, row, gamehandler.current_player, &txt);
	table_entries[row][gamehandler.current_player-1] = atoi(txt);
	
	// Reset the cell foreground to black
	gtk_clist_set_cell_style(table_clist, row, gamehandler.current_player, style_normal);
	
	return TRUE;
    }
}

// Remove all unfilled table entries and calculate and enter the sums
void
table_remove_unselected()
{
    int i,
	n = 0,
	flag = 0,
	player = gamehandler.current_player;

    g_assert(table_clist != NULL);

    // Lock the visual updates
    gtk_clist_freeze(table_clist);

    for (i=0; i<TABLE_ROWS; i++)
	if (table_entries[i][player-1] == -1)
	    gtk_clist_set_text(table_clist, i, player, "");


    // Calculate and enter the sums

    // Intermediate sum
    table_entries[INTER_SUM][player-1] = 0;
    for (i=0; i<INTER_SUM; i++)
	table_entries[INTER_SUM][player-1] += (table_entries[i][player-1] > 0 ? table_entries[i][player-1] : 0);
    table_set_cell(INTER_SUM, player, table_entries[INTER_SUM][player-1], style_normal);

    // Bonus
    if (table_entries[INTER_SUM][player-1] >= 63)
    {
	table_entries[BONUS][player-1] = 35;
	table_set_cell(BONUS, player, table_entries[BONUS][player-1], style_normal);
    }
		
    // Upper sum
    table_entries[UP_SUM][player-1] =
	table_entries[INTER_SUM][player-1] + (table_entries[BONUS][player-1] > 0 ? table_entries[BONUS][player-1] : 0);
    table_set_cell(UP_SUM, player, table_entries[UP_SUM][player-1], style_normal);
    
    // Lower sum
    table_entries[LOW_SUM][player-1] = 0;
    for (i=PASCH3; i<=CHANCE; i++)
	table_entries[LOW_SUM][player-1] += (table_entries[i][player-1] > 0 ? table_entries[i][player-1] : 0);
    table_set_cell(LOW_SUM, player, table_entries[LOW_SUM][player-1], style_normal);
	
    // Total sum
    table_entries[TOTAL_SUM][player-1] = table_entries[UP_SUM][player-1] + table_entries[LOW_SUM][player-1];
    table_set_cell(TOTAL_SUM, player, table_entries[TOTAL_SUM][player-1], style_normal);

    // Unlock the visual updates
    gtk_clist_thaw(table_clist);
	
    // Game over? All players done?
    for (i=player-1; i<MAX_PLAYERS; i++)
    {
	if (gamehandler.player_is_done[i])
	    flag ++;
	if (i < gamehandler.player_num)
	    n ++;
    }
		
    // Ok, game finally is over. And the winner is... :)
    if (flag && flag == n)
    {
	int max=0, winner=0;
	gchar *txt, *name;
	GtkWidget *mbx;

	for (i=0; i<MAX_PLAYERS; i++)
	{
	    if (i < gamehandler.player_num)
	    {
		if (table_entries[TOTAL_SUM][i] > max)
		{
		    max = table_entries[TOTAL_SUM][i];
		    winner = i;
		}
	    }
	}
	
	// Indeed, we are done. Lets give a messagebox...
	statusbar("Congratulations!");
	gtk_label_get(GTK_LABEL(table_query_label_widget(winner)), &name);
	txt = g_strdup_printf("Game over!\nAnd the winner is %s!", name);
	mbx = gnome_message_box_new(txt,
				    GNOME_MESSAGE_BOX_INFO,
				    GNOME_STOCK_BUTTON_OK,
				    NULL);
	gnome_dialog_run_and_close(GNOME_DIALOG(mbx));
	g_free(txt);
	// ... and restart the game
	gamehandler_new_game();
    }
}

void
do_debug()
{
#if 0
    int i;
    for (i=0; i<TABLE_ROWS; i++)
	g_print("*** %d -> %d\n", i, table_entries[i][0]);
#else
    int i, j;
    for (i=0; i<TABLE_ROWS-3; i++)
	for (j=0; j<TABLE_COLS; j++)
	    table_entries[i][j] = 99;
    for (i=0; i<TABLE_ROWS; i++)
	g_print("*** %d -> %d\n", i, table_entries[i][0]);
#endif
}

GtkWidget*
table_query_label_widget(int i)
{
    GtkWidget *w;

    switch (i)
    {
    case 0:  w = lookup_widget(gamehandler_query_mainwidget(), "label_player1"); break;
    case 1:  w = lookup_widget(gamehandler_query_mainwidget(), "label_player2"); break;
    case 2:  w = lookup_widget(gamehandler_query_mainwidget(), "label_player3"); break;
    default:
	w = NULL;
	g_assert_not_reached();
    }

    g_assert(w != 0);
    return w;
}

void
table_set_playernames()
{
    int i;

    g_assert(table_clist != NULL);

    g_print("playernum = %d\n", gamehandler.player_num);

    for (i=0; i<gamehandler.player_num; i++)
	gtk_label_set_text(GTK_LABEL(table_query_label_widget(i)), gamehandler.player_name[i]);

    for (; i<TABLE_COLS; i++)
	gtk_label_set_text(GTK_LABEL(table_query_label_widget(i)), "");
}

void
table_hilite_player_label()
{
    int n = gamehandler.current_player - 1;
    gtk_widget_set_style(table_query_label_widget(n), style_red);
    if (gamehandler.player_num > 1)
	gtk_widget_set_style(table_query_label_widget((--n < 0 ? gamehandler.player_num-1 : n)), style_normal);
}
